#!/usr/bin/env python

#AXIOME2 main intro function

from .modules import AxiomeAnalysis
from os.path import dirname, abspath
import argparse
import ui
import utils as util

source_dir = dirname(abspath(__file__))

def main():
    #Argument handling
    parser = argparse.ArgumentParser(prog='axiome')
    subparsers = parser.add_subparsers(title='subcommands',description='the following subcommands are possible: ui, process, utility', dest='subparser_name')
    ui_parser = subparsers.add_parser("ui")
    ui_parser.add_argument("-i", metavar="input", help="Optional .ax file to edit", type=str)
    process_parser = subparsers.add_parser("process")
    process_parser.add_argument("-i", metavar="input", help=".ax file to process", required=True, type=str)
    utility_parser = subparsers.add_parser("utility")
    utility_parser.add_argument("action", choices=["mapping_template", "sample_data"], help="mapping_template: generates a file mapping template in the current directory, which can be opened in a spreadsheet program; sample_data: copies AXIOME sample data into the current directory")
    args = parser.parse_args()

    #Argparse ensures that we meet the requirements
    if args.subparser_name == "ui":
        App = ui.AXIOMEUI(args.i)
        App.run()
    elif args.subparser_name == "process":
        axiome_analysis = AxiomeAnalysis(args.i)
        working_directory = axiome_analysis.working_directory
        print "AXIOME file processed\nDirectory '%s' created\nRun command `make` in directory '%s'" % (working_directory, working_directory)
    elif args.subparser_name == "utility":
        if args.action == "sample_data":
            util.copySampleAxData()
            print "Successfully copied sample.ax and sample_file_mapping.tsv to current directory\nRun `axiome process -i sample.ax` to process sample file"
        elif args.action == "mapping_template":
            util.generateMappingTemplate(AxiomeAnalysis(None))
            
if __name__ == "__main__":
    main()



#!/usr/bin/env python

import xml.dom.minidom as xml
from os import listdir, mkdir
from os.path import dirname, isfile, splitext, isdir, abspath
import re

#Finds the install directory, from which the res folder can be found
source_dir = dirname(abspath(__file__))

def getWorkflowList():
    master_file = xml.parse(source_dir+"/res/modules/master.xml")
    workflow_list = list()
    for workflow in master_file.getElementsByTagName("workflow"): 
        workflow_list.append(workflow.getAttribute("name"))
    return workflow_list

class AxiomeAnalysis(object):
    def __init__(self, ax_file, workflow = None):
        """AxiomeAnalysis: Class that controls loading and activating
        of modules, creation of resulting Makefile and report
        """
        #An AxAnalysis can be initiated with or without an AXIOME file
        #Without an AXIOME file, we just load all of the plugins
        #This is good for tests, and generating the UI
        if ax_file:
            try:
                self.ax_file = xml.parse(ax_file)
            except:
                raise StandardError, "Error parsing XML file %s" % ax_file
            #Set the working directory to the .ax file location
            #but add axiome as the extension
            self.working_directory = splitext(ax_file)[0] + ".axiome"
            try:
                if not isdir(self.working_directory):
                    mkdir(self.working_directory)
            except OSError:
                raise OSError, "Error: Could not create axiome directory"
            self.makefile = AxMakefile(self.working_directory+"/Makefile")
        else:
            self.working_directory = ""
            self.ax_file = None
        try:
            self.master_file = xml.parse(source_dir+"/res/modules/master.xml")
        except:
            raise StandardError, "Error parsing XML file %s" % master_file
        #File manifest
        #Contains the filenames, and the module and active submodule that owns it
        self._manifest = {}
        #Load the modules, which also loads all submodules
        if workflow:
            self.workflow = workflow
            self._modules = self.loadModules(self.workflow)
        else:
            self.workflow = self.getWorkflow()
            self._modules = self.loadModules(self.workflow)
        #Now load the actual modules in the .ax file
        #But only if the ax file was given
        if ax_file:
            self.report = AxReport(self.working_directory+"/report.html")
            self._activated_submodules = self.activateSubmodules()
            for active_submodule in self._activated_submodules:
                active_submodule._submodule._process.createMakefileString(active_submodule)
            self.report.writeHTML()
            self.makefile.writeMakefile()

    def getModuleByName(self, name):
        """Searches through initiated modules and returns module matching
        the given name
        """
        #Go through the modules and get it by its name
        for module in self._modules:
            if module.name == name:
                return module
        raise ValueError, "Module %s not found" % name
    
    def getWorkflow(self):
        if self.ax_file:
            workflow_list = getWorkflowList()
            workflow = self.ax_file.getElementsByTagName("axiome")[0].getAttribute("workflow")
            if workflow not in workflow_list:
                raise ValueError, "Workflow in loaded .ax file not found in master.xml"
            else:
                return workflow
        else:
            return "Default"
    
    def loadModules(self, workflow_name):
        """Iterates through workflow, loading all modules by the given
        name. Assumes that the module name exists as a folder in the res
        subfolder.
        """
        for workflow in self.master_file.getElementsByTagName("workflow"): 
            if workflow.getAttribute("name") == workflow_name:
            #Load the correct workflow
                module_list = []
                for module in workflow.childNodes:
                    if module.nodeType == xml.Node.ELEMENT_NODE:
                        #Get the attributes
                        args = {}
                        for i in range(0,module.attributes.length):
                            args[module.attributes.item(i).name] = module.attributes.item(i).value
                        module_list.append(AxModule(self, module.nodeName, args))
        return module_list
        
    def activateSubmodules(self):
        """Activates submodules. This means that the submodules listed
        in the .ax file will be called upon, and initiated submodules
        will be used to check if requirements are met, and to build
        the Makefile.
        """
        activated_submodules = []
        for node in self.ax_file.getElementsByTagName("axiome").item(0).childNodes:
            if node.nodeType == xml.Node.ELEMENT_NODE:
                module_name = node.nodeName
                submodule_name = node.getAttribute("method")
                submodule = self.getModuleByName(module_name).getSubmoduleByName(submodule_name)
                args = {}
                #Form a dict out of the arguments supplied
                for i in range(0,node.attributes.length):
                    #method is a special attribute that defines submodule name
                    if node.attributes.item(i).name != "method":
                        args[node.attributes.item(i).name] = node.attributes.item(i).value
                activated_submodules.append(AxActiveSubmodule(submodule, args))
        return activated_submodules
        
    def getActiveSubmodulesByModuleName(self, name):
        active_submodule_list = list()
        for submodule in self._activated_submodules:
            if submodule._submodule._module.name == name:
                active_submodule_list.append(submodule)
        if not active_submodule_list:
            raise ValueError, "No activated submodules from module '%s' found." % name
        return active_submodule_list
                
    def getActiveSubmodulesBySubmoduleName(self, module_name, submodule_name):
        active_submodule_list = list()
        for submodule in self._activated_submodules:
            if (submodule._submodule.name == submodule_name) & (submodule._submodule._module.name == module_name):
                active_submodule_list.append(submodule)
        if not active_submodule_list:
            raise ValueError, "No activated submodules from submodule '%s' found." % name
        return active_submodule_list

    def getAxFileComments(self):
        comment_list = list()
        for node in self.ax_file.getElementsByTagName("axiome").item(0).childNodes:
            if node.nodeType == xml.Node.COMMENT_NODE:
                comment_list.append(node.data)
        return comment_list


class AxMakefile(object):
    def __init__(self, makefile):
        self._file = open(makefile, "w")
        #Start with a sha-bang declaration
        #**TODO** More informative preamble (date, AXIOME2 version, etc)
        #Build up this file process by process
        self.headerString = "#!/usr/bin/make -f\n#Generated by AXIOME2\n\n"
        #Tabula rasa
        self.makefileString = ""
        #"all" string is a phony file that will allow us to simply run "make"
        #to create everything
        self.allString = "all:"
        
    def __del__(self):
        self._file.close()

    def addProcess(self, input_file_list, output_file_list, command_string):
        #Add the input/output requirements
        self.makefileString += "\n\n" + " ".join(output_file_list) + ": " + " ".join(input_file_list)
        #Add the command string
        self.makefileString += "\n" + command_string
        #Add to the phony all string
        for output_file in output_file_list:
            self.allString += " " + output_file
        
    def writeMakefile(self):
        self._file.write(self.headerString + self.allString + self.makefileString + "\n\n" + ".PHONY: all")
        
#Class which generates the output report file for AXIOME
#For now, this is going to be a simple html file
class AxReport(object):
    def __init__(self, html_file):
        #Store the page as an html string
        self.html_editing = True
        self.html_file = html_file
        self.html_string = "<!DOCTYPE HTML>\n<html>\n<head>\n"
        self.html_string += "<title>AXIOME2 Analysis Results Table of Contents</title>\n"
        self.html_string += "</head>\n<body>\n<h1>Table of Contents</h1>\n"
        self.html_string += "<h4>File links are not active until AXIOME run is complete</h4>\n<ul>\n"
        
    def addToReport(self, label, location):
        self.html_string += "<li><a href=\"" + location + "\">" + label + "</a></li>\n"
        
    def writeHTML(self):
        #Close the HTML document
        self.html_string += "</ul>\n</body>\n</html>"
        self.html_editing = False
        with open(self.html_file, 'w') as html_out:
            html_out.write(self.html_string)


#An active submodule is one that is currently in use
class AxActiveSubmodule(object):
    """Class for the "activated" submodules. Activated means that the
    user has chosen the submodule in their .ax file, and we need to load
    a specific version with the given arguments
    """
    def __init__(self, submodule, args):
        self._submodule = submodule
        #Set the arguments by filling in the defaults, where possible
        self._args = self.fillDefaults(args, submodule)
        #Set a designation for the submodule, for use in making directories
        self.name = self._submodule._module.name + "/" +self._submodule.name
        if self._submodule._module._value["multi"]:
            self.name += str(self._submodule.num_loaded)
            self._submodule.num_loaded += 1
        #Check if the requirements of the module are met
        if not self._submodule._input.requirementsMet(args):
            raise ValueError, "Requirements for submodule '%s', module '%s' not met. Exiting." % (self._submodule.name, self._submodule._module.name)
            
    def fillDefaults(self, args, submodule):
        #Go into the requirements for the submodule
        input_values = self._submodule._input._values
        for item in input_values:
            if item["name"] not in args.keys():
                #If a default exists for it, use it
                if "default" in input_values:
                    args[item]=input_values[item]["default"]
        return args

class AxModule(object):
    def __init__(self, analysis, module_name, args):
        """Class for initiated modules. Its job is to hold the properties
        of the module and a list of its initiated submodules.
        """
        self.name = module_name
        self._analysis = analysis
        #Default properties
        self._value = {"required":False, "multi":False, "label":self.name, "default":list(), "help":""}
        self.updateProperties(args)
        self._submodules = self.loadSubModules()
    
    def updateProperties(self, args):
        for prop in args:
            if prop in ["required", "multi"]:
                if args[prop].lower() in ["true","t"]:
                    self._value[prop] = True
                else:
                    self._value[prop] = False
            elif prop in ["label"]:
                self._value["label"] = args["label"]
            elif prop in ["default"]:
                self._value["default"] = args["default"].replace(" ","").split(",")
            elif prop in ["help"]:
                self._value["help"] = args["help"]
    
    def getSubmoduleByName(self, name):
        #Go through the submodules and get it by its name
        for submodule in self._submodules:
            if submodule.name == name:
                return submodule
        raise ValueError, "Submodule %s not found in module" % (name, self.name)
    
    def loadSubModules(self):
        #Go into the module folder and load all of the submodules
        submodule_list = []
        for submodule in listdir(source_dir + "/res/modules/%s/" % self.name):
            #**TODO** Need to check if it is a file and a .xml suffix
            try:
                xml_obj = xml.parse(source_dir + "/res/modules/%s/%s" % (self.name, submodule))
            except:
                raise StandardError, "Could not parse file '" + source_dir + "/res/modules/%s/%s'" % (self.name, submodule)
            submodule_list.append(AxSubmodule(self, xml_obj))
        return submodule_list
    
class AxSubmodule(object):
    def __init__(self, module, xml_obj):
        self._module = module
        self.name = xml_obj.getElementsByTagName("plugin").item(0).getAttribute("name")
        #Go through the submodule, creating the AxInput AxProcess and AxInfo objects
        self._input = AxInput(self, xml_obj.getElementsByTagName("input"))
        self._process = AxProcess(self, xml_obj.getElementsByTagName("process"))
        self._info = AxInfo(self, xml_obj.getElementsByTagName("info"))
        self.num_loaded = 0

class AxInput(object):
    def __init__(self, submodule, xml_obj):
        #Give access to the submodule that originates this object
        self._submodule = submodule
        #Structure that stores all pertinent data
        self._values = list()
        #xml_obj is the <input> sections (including <input> tags)
        #There should only be one of these
        #Populate the values from the XML object
        for node in xml_obj.item(0).childNodes:
            if node.nodeType == xml.Node.ELEMENT_NODE:
                #Get all of the information
                data_type = node.nodeName
                name = node.getAttribute("name")
                label = node.getAttribute("label")
                required = node.getAttribute("required")
                if required.lower() in ["true","t"]:
                    required = True
                else:
                    required = False
                default = node.getAttribute("default")
                value_dict = {"name":name, "type":data_type, "label":label, "required":required, "default":default}
                #Optional information for int and float types
                if data_type in ["int","float"]:
                    minimum = node.getAttribute("min")
                    maximum = node.getAttribute("max")
                    if not default:
                        if float(minimum) > 0:
                            default = minimum
                        else:
                            default = "0"
                    value_dict["min"]=minimum
                    value_dict["max"]=maximum
                self._values.append(value_dict)
                #**TODO** Check if label, name required, min/max required if int/float
                
    def requirementsMet(self, args):
        #Get a list of all required arguments
        required = [ d["name"] for d in self._values if d["required"] ]
        names = [ d["name"] for d in self._values ]
        #Complain if a required argument is missing
        for item in required:
            if item not in args:
                print "Error: Required item %s not in definition" % item
                return False
        for item in args:
            if (item not in names) & (item != "axiome_submodule"):
                print "Warning: Unused attribute %s" % item
                return False
        return True
        #**TODO** Check data type and make sure it follows requirements
        #ie, files exist, numbers in range
        
    def getValuesForInput(self, name):
        for value_dict in self._values:
            if value_dict["name"] == name:
                return value_dict
        raise ValueError, "Failed to find input name '%s' in submodule '%s'" % (name, self._submodule.name)
    
class AxProcess(object):
    def __init__(self, submodule, xml_obj):
        #Give access to the submodule that originates this object
        self._submodule = submodule
        #Structure that stores all pertinent data
        self._values = []
        #xml_obj is the <process> sections (including <process> tags)
        #There can be more than one of these
        #Populate the values from the XML object
        for process in xml_obj:
            process_dict = {"input":[],"output":{}, "command":{}}
            for node in process.childNodes:
                if node.nodeType == xml.Node.ELEMENT_NODE:
                    if node.nodeName == "input":
                        process_dict["input"].append(node.getAttribute("name"))
                    elif node.nodeName == "output":
                        process_dict["output"][node.getAttribute("name")] = {}
                        process_dict["output"][node.getAttribute("name")]["report_label"] = node.getAttribute("report_label")
                        process_dict["output"][node.getAttribute("name")]["report_variable"] = node.getAttribute("report_variable")
                    else:
                        process_dict["command"]["label"] = node.getAttribute("label")
                        process_dict["command"]["cmd"] = node.getAttribute("cmd")
                        process_dict["command"]["format"] = node.getAttribute("format")
                        process_dict["command"]["input"] = node.getAttribute("input")
                        process_dict["command"]["output"] = node.getAttribute("output")
                        process_dict["command"]["variable"] = node.getAttribute("variable")
            self._values.append(process_dict)

    def createMakefileString(self, active_submodule):
        #Steps:
        # For each process:
        # - make input/output string
        # - make command string
        for process in self._values:
            input_file_list = list()
            for input_file in process["input"]:
                #Check for variables
                resolved_input_file_list = self.resolve_variable(input_file, active_submodule)
                #Now we need to check if the file exists
                for resolved_input_file in resolved_input_file_list:
                    if not isfile(resolved_input_file):
                        #If it does not, see if it exists in the file manifest
                        if resolved_input_file not in self._submodule._module._analysis._manifest:
                            #If it does not exist in the manifest or on system, complain
                            raise ValueError, "Required input file '%s' called by submodule '%s' does not exist on system and not created by previously called module." % (input_file, active_submodule.name)
                        else:
                            input_file_list.append(self._submodule._module._analysis._manifest[resolved_input_file] + "/" + resolved_input_file)
                    else:
                        input_file_list.append(resolved_input_file)
            output_file_list = list()
            for output_file, output_file_dict in process["output"].iteritems():
                #Check for variables
                resolved_output_file_list = self.resolve_variable(output_file, active_submodule)
                for resolved_output_file in resolved_output_file_list:
                    #Add files to manifest, but don't update:
                    if resolved_output_file not in self._submodule._module._analysis._manifest:
                        self._submodule._module._analysis._manifest[resolved_output_file] = active_submodule.name
                    output_file_list.append(active_submodule.name + "/" + resolved_output_file)
                    if output_file_dict["report_label"]:
                        label = output_file_dict["report_label"]
                        variable = output_file_dict["report_variable"]
                        sep = ", "
                        resolved_variable_list = self.resolve_variable(variable, active_submodule)
                        label = label.replace("${v}",sep.join(resolved_variable_list),1)
                        self._submodule._module._analysis.report.addToReport(label, active_submodule.name + "/" + resolved_output_file)

            #Now resolve the command
            command_str = process["command"]["cmd"]
            if process["command"]["format"] in ["slist"]:
                sep = " "
            elif process["command"]["format"] in ["sclist"]:
                sep = ";"
            else:
                sep = ","
            #First, the input variables:
            for input_file in process["command"]["input"].split(","):
                resolved_input_file_list = self.resolve_variable(input_file, active_submodule)
                cmd_input_file_list = list()
                for resolved_input_file in resolved_input_file_list:
                    if not isfile(resolved_input_file):
                        #If it does not, see if it exists in the file manifest
                        if resolved_input_file not in self._submodule._module._analysis._manifest:
                            #If it does not exist in the manifest or on system, complain
                            raise ValueError, "Required input file '%s' does not exist on system and not created by previously called module." % input_file
                        else:
                            cmd_input_file_list.append(self._submodule._module._analysis._manifest[resolved_input_file] + "/" + resolved_input_file)
                    else:
                        cmd_input_file_list.append(resolved_input_file)
                command_str = command_str.replace("${i}", sep.join(cmd_input_file_list),1)
            #Then the output variables:
            for output_file in process["command"]["output"].split(","):
                resolved_output_file_list = self.resolve_variable(output_file, active_submodule)
                cmd_output_file_list = list()
                for resolved_output_file in resolved_output_file_list:
                    cmd_output_file_list.append(active_submodule.name + "/" + resolved_output_file)
                    #Update ownership in manifest
                    self._submodule._module._analysis._manifest[resolved_output_file] = active_submodule.name
                command_str = command_str.replace("${o}",sep.join(cmd_output_file_list),1)
            #Finally, the variables
            for variable in process["command"]["variable"].split(","):
                resolved_variable_list = self.resolve_variable(variable, active_submodule)
                command_str = command_str.replace("${v}",sep.join(resolved_variable_list),1)
            command_str = "\tmkdir -p %s/\n\t%s" % (active_submodule.name,command_str)
            self._submodule._module._analysis.makefile.addProcess(input_file_list, output_file_list, command_str)
            
            

    #Probably the most complex method in this entire piece of software
    def resolve_variable(self, in_variable, active_submodule):
        #First, find the variables that we need to resolve
        variables = find_variables(in_variable)
        if not variables:
            return [in_variable]
        replacement_dict={}
        for variable in variables:
            #Take out the ${} from the variable
            variable = variable[2:-1]
            #Input variable resolution:
            #First check if it is a local or global variable
            #If local, get the local submodule and look for the variable there
            if variable.count(":") == 0:
                #Local submodule
                #Look to see if the variable is in the active submodule's argument list
                if variable in active_submodule._args:
                    replacement_dict[variable] = [active_submodule._args[variable]]
                elif variable == "PWD":
                    replacement_dict[variable] = [active_submodule.name]
                elif not active_submodule._submodule._input.getValuesForInput(variable)["required"]:
                    replacement_dict[variable] = ""
                else:
                    raise ValueError, "Cannot resolve variable %s, not found in submodule %s, module %s" % (variable, self._submodule.name, self._submodule._module.name)   
            else:
                #Global submodule
                module_name = variable.split(":")[0]
                variable = variable.split(":")[1]
                active_submodules = self._submodule._module._analysis.getActiveSubmodulesByModuleName(module_name)
                resolved_variables = list()
                for submodule in active_submodules:
                    #Look to see if the variable is in the active submodule's argument list
                    if variable in submodule._args:
                        resolved_variables.append(submodule._args[variable])
                replacement_dict[module_name+":"+variable] = resolved_variables
        #Check if the replacement lists are all the same size
        replace_max = max(map(len, replacement_dict.values()))
        #A variable should only be found in 1 module, or all activated submodules
        #Any other combination is too difficult to deal with in an automated fashion
        if replace_max != 1:
            for variable, replacement_list in replacement_dict.iteritems():
                if len(replacement_list) == 1:
                    #This recycles values that are 
                    replacement_dict[variable] = replacement_list*replace_max
                elif len(replacement_list) != replace_max:
                    raise ValueError, "Replacement list not 1 or equal to the replacement max"
        replacement_list = apply(zip, replacement_dict.values())
        to_replace = replacement_dict.keys()
        replaced_strings = list()
        for i in range(0, len(replacement_list)):
            for j in range(0, len(to_replace)):
                if replacement_list[i][j] == "":
                    replacement_string = ""
                else:
                    replacement_string = in_variable.replace("${"+to_replace[j]+"}", replacement_list[i][j])
            replaced_strings.append(replacement_string)
        return replaced_strings
    
class AxInfo(object):
    def __init__(self, submodule, xml_obj):
        #Give access to the submodule that originates this object
        self._submodule = submodule
        #Structure that stores all pertinent data
        self._values = {"input":{},"help":{"text":"No description given."}}
        #xml_obj is the <input> sections (including <input> tags)
        #There should only be one of these
        #Populate the values from the XML object
        for node in xml_obj.item(0).childNodes:
            if node.nodeType == xml.Node.ELEMENT_NODE:
                if node.nodeName == "command":
                #Get all of the information
                    label = node.getAttribute("label")
                    cmd = node.getAttribute("cmd")
                    self._values["command"] = {"label":label, "cmd":cmd}
                elif node.nodeName == "help":
                    text = node.getAttribute("text")
                    self._values["help"] = {"text":text}
                elif node.nodeName == "input":
                    name = node.getAttribute("name")
                    text = node.getAttribute("text")
                    self._values["input"][name] = text
                    
    def createMakefileString(self):
        makefile_string = ""
        for label, cmd in self._values:
            makefile_string += "\t@echo \"%s\"\n\t%s >> versions.log\n"
        return makefile_string

#Find variables in a string
def find_variables(source_str):
    #Pull out everything that is inside a "${}"
    return re.findall("\$\{.*?\}",source_str)



from os import getcwd
from os.path import dirname, abspath
from shutil import copyfile
from collections import OrderedDict

def copySampleAxData():
    '''Copies the sample AXIOME data, updating the filepaths in the process'''
    #Finds the install directory, from which the res folder can be found
    cwd = getcwd()
    source_dir = dirname(abspath(__file__))
    sample_file = source_dir + "/res/sample/sample.ax"
    with open(sample_file, "r") as sample_in:
        with open("sample.ax", "w") as sample_out:
            for line in sample_in:
                sample_out.write(line.replace("@SOURCE_DIR@",source_dir).replace("@PWD@",getcwd()))
    mapping_file = source_dir + "/res/sample/sample_file_mapping.tsv"
    with open(mapping_file, "r") as mapping_in:
        with open("sample_file_mapping.tsv", "w") as mapping_out:
            for line in mapping_in:
                mapping_out.write(line.replace("@SOURCE_DIR@",source_dir))

def generateMappingTemplate(AxAnalysis):
    '''Generates a file mapping template based on loaded source submodules'''
    source_module = AxAnalysis.getModuleByName("source")
    source_inputs = OrderedDict()
    for submodule in source_module._submodules:
        AxIn = submodule._input
        for requirement in AxIn._values:
            if requirement["name"] not in source_inputs:
                source_inputs[requirement["name"]] = [(submodule.name, requirement["label"], requirement["required"])]
            else:
                source_inputs[requirement["name"]].append((submodule.name, requirement["label"], requirement["required"]))
    #Now we output this information into a tab separated value file
    with open("axiome_file_mapping_template.tsv", "w") as template:
        template.write("#File mapping template generated by AXIOME2\n")
        template.write("#Instructions:\n")
        template.write("# - sample_alias column contains UNIQUE sample names and must match sample metadata sheet\n")
        template.write("# - axiome_module column contains the name of the AXIOME module that will process the input file\n")
        template.write("# - all other columns are variables requested by AXIOME source modules, and only those columns required by the chosen module need to be filled out\n")
        source_module_string = "#Source module (module that requires this variable)\t"
        description_string = "#Description of variable\t"
        required_string = "#Required?\t"
        column_headers = "sample_alias\taxiome_submodule"
        for variable_name, submodules in source_inputs.iteritems():
			#We want sample_alias to be added in manually. It is mandatory.
			if variable_name != "sample_alias":
				source_add = []
				description_add = []
				required_add = []
				column_headers_add = []
				for info in submodules:
					source_add.append(info[0])
					description_add.append("%s: %s" % (info[0], info[1]))
					required_add.append("%s: %s" % (info[0], info[2]))
				source_module_string += "\t%s" % ", ".join(source_add)
				description_string += "\t%s" % "; ".join(description_add)
				required_string += "\t%s" % "; ".join(required_add)
				column_headers += "\t%s" % variable_name
        template.write("%s\n" % source_module_string)
        template.write("%s\n" % description_string)
        template.write("%s\n" % required_string)
        template.write("%s\n" % column_headers)
        print "Successfully written file 'axiome_file_mapping_template.tsv'"
        print "This file can be opened and edited in a spreadsheet application (eg. Excel)"



#!/usr/bin/env python
# encoding: utf-8

import argparse
import npyscreen as nps
from .modules import AxiomeAnalysis, getWorkflowList
from os.path import dirname, abspath, isfile, isdir, exists
from os import getcwd, makedirs

source_dir = dirname(abspath(__file__))

class AXIOMEUI(nps.NPSAppManaged):
    def __init__(self, ax_file = None, *args, **keywords):
        self.ax_file = ax_file
        self.submodules_loaded = False
        self.submodule_forms_data = list()
        self.source_definitions = list()
        self.AxAnal = None
        super(AXIOMEUI, self).__init__(*args, **keywords)

    def onStart(self):
        #Load modules and submodules:
        self.current_page = 0
        self._display_pages = []
        nps.FIX_MINIMUM_SIZE_WHEN_CREATED = False
        self.registerForm("MAIN", IntroForm(parentApp=self))
        self.registerForm("SAVE", SaveForm(parentApp=self))
         
    def buildSubmoduleForms(self, selected_submodules):
        new_submodule_forms_data = list()
        for names in selected_submodules:
            module_name = names["module_name"]
            submodule_name = names["submodule_name"]
            if self.submodules_loaded | (not self.ax_file):
                #For each selected module+submodule, check if it is in the list
                inserted = False
                for form in self.submodule_forms_data:
                    if (module_name == form["module_name"]) and (submodule_name == form["submodule_name"]):
                        new_submodule_forms_data.append(form)
                        inserted = True
                #If not found, make a new dictionary to hold information
                if not inserted:
                    form = self.createSubmoduleForm(module_name, submodule_name, 0)
                    if form:
                        new_submodule_forms_data.append({"module_name":module_name, "submodule_name":submodule_name, "form":form, "copy_number":0, "registered":False})
            else:
                #On first load, if a .ax file was specified, load the submodules from there
                submodule_form_list = self.createSubmoduleFormFromAxFile(module_name, submodule_name)
                copy = 0
                for form in submodule_form_list:
                    new_submodule_forms_data.append({"module_name":module_name, "submodule_name":submodule_name, "form":form, "copy_number":copy, "registered":False})
                    copy += 1
        #Clear the page display list
        self._display_pages = list()
        for form_data in new_submodule_forms_data:
            formid = form_data["module_name"]+"_"+form_data["submodule_name"]+"_"+str(form_data["copy_number"])
            self.registerForm(formid,form_data["form"])
            self._display_pages.append(formid)
        self.submodule_forms_data = new_submodule_forms_data
        self.submodules_loaded = True
            
    def createSubmoduleForm(self, module_name, submodule_name, copy_number):
        #Grab the AxInput for the submodule
        module = self.AxAnal.getModuleByName(module_name)
        if not module:
            raise ValueError, "Cannot find module: %s" % module_name
        submodule = module.getSubmoduleByName(submodule_name)
        if not submodule:
            raise ValueError, "Cannot find module: %s" % submodule_name
        AxInput = submodule._input
        if not AxInput._values:
            return False
        #Create the form
        form = SubmoduleForm(module, submodule, copy_number, parentApp=self)
        if module._value["multi"]:
            form.add_widget_intelligent(nps.TitleFixedText, w_id="submodule_title_"+submodule.name, name="Submodule: %s (%s)" % (submodule.name, str(copy_number+1)))
        else:
            form.add_widget_intelligent(nps.TitleFixedText, w_id="submodule_title_"+submodule.name, name="Submodule: %s" % submodule.name)
        for requirement in AxInput._values:
            widget_type = requirement["type"]
            if widget_type == "float":
                form.add_widget_intelligent(TitleFloatSlider, w_id=requirement["name"], out_of=float(requirement["max"]), lowest=float(requirement["min"]), name=requirement["label"]+":", value=float(requirement["default"]), step=0.01)
            elif widget_type == "int":
                form.add_widget_intelligent(TitleFloatSlider, w_id=requirement["name"], out_of=int(requirement["max"]), lowest=int(requirement["min"]), name=requirement["label"]+":", value=int(requirement["default"]), step=1)
            elif widget_type == "text":
                form.add_widget_intelligent(nps.TitleText, w_id=requirement["name"], name=requirement["label"]+":", value=requirement["default"], max_height=3)
            elif widget_type == "file":
                form.add_widget_intelligent(nps.TitleFilename, w_id=requirement["name"], name=requirement["label"]+":", max_height=3)
            required = requirement["required"]
            form._value["input"].append({"name":requirement["name"],"required":required, "type":requirement["type"]})
        #If you can have multiple submodules, then add an add/remove button
        #to duplicate submodules
        if module._value["multi"]:
            form.nextrely+=1
            form.add_widget_intelligent(AddFormButton, name="Add Copy of Submodule", w_id="submodule_add")
            form.add_widget_intelligent(RemoveFormButton, name="Remove Copy of Submodule", w_id="submodule_remove")
        #Add in description and help text in a box
        form.nextrely+=1
        AxInfo = submodule._info
        description_box = form.add_widget_intelligent(TitleExitPager, name="Submodule Description:", slow_scroll = True, max_height = 4, scroll_exit = True, autowrap = True, values = [AxInfo._values["help"]["text"]])
        help_text = list()
        for name, text in AxInfo._values["input"].iteritems():
            label = AxInput.getValuesForInput(name)["label"]
            help_text.append("%s: %s" % (label, text))
        if help_text:
            help_box = form.add_widget_intelligent(TitleExitPager, name="Argument Descriptions:", max_height=6, slow_scroll = True, scroll_exit = True, autowrap = True, values = help_text)
        return form
        
    def createSubmoduleFormFromAxFile(self, module_name, submodule_name):
        #Get the activated submodules
        #If there is a required step, but it isn't in the AxFile, it'll cause a crash
        #If we can't find an active submodule, make a new one
        try:
            active_submodule_list = self.AxAnal.getActiveSubmodulesBySubmoduleName(module_name, submodule_name)
        except:
            active_submodule_list = None
        submodule_form_list = list()
        copy = 0
        if active_submodule_list:
            for active_submodule in active_submodule_list:
                form = self.createSubmoduleForm(module_name, submodule_name, copy)
                if form:
                    args = active_submodule._args
                    for variable, value in args.iteritems():
                        widget = form.get_widget(variable)
                        if widget.__class__.__name__ is "TitleFloatSlider":
                            value = float(value)
                        try:
                            widget.value = value
                        except:
                            raise ValueError, widget.__class__.__name__
                    copy += 1
                    submodule_form_list.append(form)
        else:
            form = self.createSubmoduleForm(module_name, submodule_name, copy)
            if form:
				submodule_form_list.append(form)
        return submodule_form_list

class IntroForm(nps.FormMultiPageAction):   
    def create(self):
        #Keep it from crashing when terminal size changes
        self.ALLOW_RESIZE = False
        self.OK_BUTTON_TEXT = "Next"
        self.CANCEL_BUTTON_TEXT = "Exit"
        self._widget_list = []
        #Intro message
        message="""Welcome to AXIOME\nTo navigate through the UI, use arrow keys or TAB.
            \nENTER will select an option, and SPACE will deselect an option.
            \nRequirements for the analysis:
            \n\t- File mapping in tab-separated spreadsheet format (.tsv)
            \n\t- Metadata mapping in tab-separated spreadsheet format (.tsv)"""
        nps.notify_confirm(message=message, title="Message", form_color='STANDOUT', wrap=True, wide=True, editw=1)
        self.name = "Select Workflow"
        #Get the possible workflows from the master.xml file
        workflow_list = getWorkflowList()
        if self.parentApp.ax_file:
            self.parentApp.AxAnal = AxiomeAnalysis(self.parentApp.ax_file)
            value = workflow_list.index(self.parentApp.AxAnal.getWorkflow())
        else:
            try:
                value = workflow_list.index("Default")
            except:
                value = 0
        self.add_widget_intelligent(nps.TitleSelectOne, name="Select Workflow", w_id="select_workflow", values=workflow_list, value=value, max_height=len(workflow_list)+2, scroll_exit=True)
        
    def on_ok(self):
        #Create the ModuleForm
        workflow = self.get_widget("select_workflow").values[self.get_widget("select_workflow").value[0]]
        #If the workflow is different, recreate it
        if self.parentApp.AxAnal:
            if self.parentApp.AxAnal.workflow != workflow:
                #Throw up a warning that all options will be overwritten
                response = nps.notify_ok_cancel("Warning: Changing workflows will reset all previously entered values (including from loaded .ax file). Do you wish to continue?", title="Warning")
                if response:
                    self.parentApp.AxAnal = AxiomeAnalysis(self.parentApp.ax_file, workflow)
                    module_form = ModuleForm(parentApp=self.parentApp)
                    self.parentApp.registerForm("MODULE",module_form)
                else:
                    self.editing = True
                    return
        else:
            self.parentApp.AxAnal = AxiomeAnalysis(self.parentApp.ax_file, workflow)
            module_form = ModuleForm(parentApp=self.parentApp)
            self.parentApp.registerForm("MODULE",module_form)
        #If the page doesn't exist (in case where AxAnal is loaded from
        # .ax file), then make it
        if "MODULE" not in self.parentApp.getHistory():
            module_form = ModuleForm(parentApp=self.parentApp)
            self.parentApp.registerForm("MODULE", module_form)
        self.parentApp.setNextForm("MODULE")
        
    def on_cancel(self):
        response = nps.notify_ok_cancel("Are you sure you want to exit? All unsaved changes will be lost.", title="Exit?", form_color='STANDOUT', wrap=True, editw=0)
        if response:
            self.parentApp.setNextForm(None)
            self.editing = False
        else:
            self.editing = True

class ModuleForm(nps.FormMultiPageAction):
    def create(self):
        #Keep it from crashing when terminal size changes
        self.ALLOW_RESIZE = False
        self.OK_BUTTON_TEXT = "Next"
        self.CANCEL_BUTTON_TEXT = "Previous"
        self._widget_list = []
        self.name = "Select Pipeline Steps"
        #Fill with each module's submodule lists
        for module in self.parentApp.AxAnal._modules:
            #Build the submodule choices
            values = []
            for submodule in module._submodules:
                values.append(submodule.name)
            if values:
                #Two options: multi or not
                if module._value["multi"]:
                    widget = nps.MultiSelect
                    value = None
                else:
                    widget = nps.SelectOne
                    value = 0
                #Special case: mapping file, we want to select a spreadsheet
                if module.name == "source":
                    help_widget = self.add_widget_intelligent(HelpButton, help_msg="Select a tab separated (.tsv) file describing each file.\nMust include the columns 'sample_alias' and 'axiome_submodule'.", name="Source File Mapping:")
                    self.nextrelx = 10
                    widget = nps.Filename
                    choice_widget = self.add_widget_intelligent(widget, w_id="module_source", value=None, max_height=4)
                    choice_widget.value = getcwd()
                    #For consistent spacing
                    self.nextrely += 1
                    self.nextrelx = 2
                else:
                    #Sort out the values, given the defaults
                    defaults = list()
                    for i in range(0, len(values)):
                        if values[i] in module._value["default"]:
                            defaults.append(i)
                    if defaults:
                        value = defaults
                    help_text = module._value["help"] + "\n"
                    if not help_text:
                        help_text = "No description given in module definition."
                    for submodule in module._submodules:
						try:
							help_text += "\n%s: %s" % (submodule.name, submodule._info._values["help"]["text"])
						except:
							raise ValueError, submodule.name
                    help_widget = self.add_widget_intelligent(HelpButton, help_msg=help_text, name="%s:" % module._value["label"])
                    self.nextrelx = 10
                    choice_widget = self.add_widget_intelligent(widget, w_id="module_"+module.name, values=values, value=value, max_height=len(values)+1, scroll_exit=True)
                    self.nextrelx = 2
                self._widget_list.append({"module_name":module.name,"widget":choice_widget})
        #If the ax_file was provided, change the default values to the
        #values in the file/AxAnal
        if self.parentApp.ax_file:
            #The used submodules are in the getActiveSubmodulesByModuleName method
            for widget in self._widget_list:
                value = None
                if widget["module_name"] == "source":
                    #Look for the comment that tells us the source file location
                    for comment in self.parentApp.AxAnal.getAxFileComments():
                        if "source mapping_file=" in comment:
                            #Parse out the path
                            value = comment.split('"')[-2]
                else:
                    value = list()
                    try:
                        submodule_list = self.parentApp.AxAnal.getActiveSubmodulesByModuleName(widget["module_name"])
                    except:
                        submodule_list = None
                    if submodule_list:
                        for active_submodule in submodule_list:
                            if active_submodule._submodule.name in widget["widget"].values:
                                index = widget["widget"].values.index(active_submodule._submodule.name)
                                if index not in value:
                                    value.append(index)
                if value:
                    widget["widget"].value = value
        
    def on_ok(self):
        source_file_widget = self.get_widget("module_source")
        source_file_path = source_file_widget.value
        if not source_file_path:
            nps.notify_confirm(message="Source file mapping required")
            self.editing = True
            return
        if not isfile(source_file_path):
            nps.notify_confirm(message="Given source file mapping is not a file.")
            self.editing = True
            return
        if not self.sourceFileCheck(source_file_path):
            nps.notify_confirm(message="Error in source file mapping")
            self.editing = True
            return
        #Collect information on the selected widgets
        selected_submodules = list()
        for widget_info in self._widget_list:
            module_name = widget_info["module_name"]
            widget = widget_info["widget"]
            if type(widget.value) is list:
                for choice in widget.value:
                    selected_submodules.append({"module_name":module_name,"submodule_name":widget.values[choice]})
        nextForm = self.parentApp.buildSubmoduleForms(selected_submodules)
        self.parentApp.current_page = 0
        self.parentApp.setNextForm(self.parentApp._display_pages[0])
       
    def on_cancel(self):
        self.parentApp.setNextForm("MAIN")
        
    def sourceFileCheck(self, source_file_path):
    #Go through each line of the sources file and verify the contents
        with open(source_file_path) as source_file:
            #Clear the old definitions out
            self.parentApp.source_definitions = list()
            column_headers = None
            for definition in source_file:
                #Ignore comment lines
                if definition[0] == "#":
                    continue
                else:
                    #We need to detect the column headers row
                    #We are using the EBI metadata sheet as our template here
                    if "sample_alias" in definition.split("\t"):
                        column_headers = definition
                    else:
                        if not column_headers:
                            return False
                        #Special column headers:
                        #axiome_submodule: name of source submodule that controls the sample
                        sample_dict = dict(zip(column_headers.strip().split("\t"), definition.strip().split("\t")))
                        try:
                            submodule_name = sample_dict["axiome_submodule"]
                        except KeyError:
                            raise KeyError, "Required column header 'axiome_submodule' not found"
                        #Get the submodule
                        submodule = self.parentApp.AxAnal.getModuleByName("source").getSubmoduleByName(submodule_name)
                        #Get the AxInput object
                        AxInput = submodule._input
                        #Make sure the requirements are met
                        if AxInput.requirementsMet(sample_dict):
                            #Store the dict
                            self.parentApp.source_definitions.append(sample_dict)
                        else:
                            raise ValueError, str(sample_dict)
                            #**TODO** Make this a persistent popup warning when trying to exit
                            #the intro form, not a raised error
                            #return False
            if not self.parentApp.source_definitions:
                return False
            return True
            
class SaveForm(nps.FormMultiPageAction):
    def __init__(self, *args, **keywords):
        super(SaveForm, self).__init__(*args, **keywords)
        
    def create(self):
        self.name = "Save AXIOME File"
        self.ALLOW_RESIZE = False
        self.OK_BUTTON_TEXT = "Save"
        self.CANCEL_BUTTON_TEXT = "Previous"
        self.add_widget_intelligent(nps.TitleFilename, w_id="save_filename", name="Save File Location...", max_height=3)
        
    def on_cancel(self):
        #Go back to the last page
        self.parentApp.current_page -= 1
        self.parentApp.setNextForm(self.parentApp._display_pages[self.parentApp.current_page])
        
    def on_ok(self):
        #Get the save file path
        file_name = self.get_widget("save_filename").value
        if not file_name:
            nps.notify_wait("Save File Location must be specified.", title="Error", form_color='STANDOUT', wrap=True, wide=True)
            self.editing = True
            return
        if isdir(file_name):
            nps.notify_wait("Save File Location must not be a directory.", title="Error", form_color='STANDOUT', wrap=True, wide=True)
            self.editing = True
            return
        #Options are OK. Now write the file:
        directory = dirname(abspath(file_name))
        if not exists(directory):
            makedirs(directory)
        with open(file_name,'w') as out_ax:
            #Start with the XML header
            #We are manually writing XML, because that's how I roll
            ax_file_string = '<?xml version="1.0"?>\n<axiome workflow="%s">\n' % self.parentApp.AxAnal.workflow
            submodules = list()
            for widget_info in self.parentApp.getForm("MODULE")._widget_list:
                module_name = widget_info["module_name"]
                #Get the selection(s) from the selection widget
                widget = widget_info["widget"]
                selections = widget.value
                if module_name != "source":
                    for choice in selections:
                        submodules.append([module_name, widget.values[choice]])
                else:
                    submodules.append(["source",None])
            #For each module, go through the submodule page list to find a match
            for submodule in submodules:
                module_name = submodule[0]
                submodule_name = submodule[1]
                #Special case: source
                if module_name == "source":
                    mapping_file = self.parentApp.getForm("MODULE").get_widget("module_source").value
                    ax_file_string += '\t<!--source mapping_file="%s"-->\n' % mapping_file
                    ax_file_string += self.file_mapping_to_ax(mapping_file)
                else:
                    found = False
                    for submodule_form in self.parentApp.submodule_forms_data:
                        if (submodule_form["module_name"] == module_name) & (submodule_form["submodule_name"] == submodule_name):
                            found = True
                            #If it matches, write a line for it
                            def_string = ""
                            requirements = submodule_form["form"]._value
                            for item in requirements["input"]:
                                #Get the widget and its value
                                widget_id = item["name"]
                                widget_value = submodule_form["form"].get_widget(widget_id).value
                                if widget_value:
                                    def_string += ' %s="%s"' % (widget_id, widget_value)
                            ax_file_string += '\t<%s method="%s"%s/>\n' % (module_name, submodule_name, def_string)
                    if not found:
                        ax_file_string += '\t<%s method="%s"/>\n' % (module_name, submodule_name)
            ax_file_string += "</axiome>"
            out_ax.write(ax_file_string)
        response = nps.notify_ok_cancel("File saved successfully to %s. Do you want to exit?" % (file_name), title="Saved!", form_color='STANDOUT', wrap=True, editw=0)
        if response:
            self.parentApp.setNextForm(None)
            self.editing = False
        else:
            self.editing = True
 
    def file_mapping_to_ax(self, mapping_file):
        with open(mapping_file) as mapping:
            #Clear the old definitions out
            self.parentApp.source_definitions = list()
            column_headers = None
            source_defs = ""
            for definition in mapping:
                #Ignore comment lines
                if definition[0] == "#":
                    continue
                else:
                    #We need to detect the column headers row
                    #We are using the EBI metadata sheet as our template here
                    if "sample_alias" in definition.split("\t"):
                        column_headers = definition
                    else:
                        if not column_headers:
                            return False
                        #Start building the line
                        source_variables = ""
                        sample_dict = dict(zip(column_headers.strip().split("\t"), definition.strip().split("\t")))
                        for header, value in sample_dict.iteritems():
                            if (value != "") & (header != 'axiome_submodule'):
                                source_variables += ' %s="%s"' % (header, value)
                        source_line = '\t<source method="%s"%s/>\n' % (sample_dict["axiome_submodule"], source_variables)
                        source_defs += source_line
        return source_defs
                        
        
class SubmoduleForm(nps.FormMultiPageAction):
    def __init__(self, module, submodule, copy_number, *args, **keywords):
        #Keep it from crashing when terminal size changes
        self.ALLOW_RESIZE = False
        self.module = module
        self.submodule = submodule
        self.copy_number = copy_number
        #Contain the important information about the form
        self._value = {"module_name":module.name, "submodule_name":submodule.name, "input":[]}
        super(SubmoduleForm, self).__init__(*args, **keywords)
    
    def on_cancel(self):
        if self.parentApp.current_page > 0:
            self.parentApp.current_page -= 1
        else:
            self.parentApp.current_page = -1
    
    def on_ok(self):
        #Validate the input variables
        #Get the variables
        args = {}
        for input_item in self._value["input"]:
            name = input_item["name"]
            if self.get_widget(name).value:
                args[name] = self.get_widget(name).value
        if not(self.submodule._input.requirementsMet(args)):
            nps.notify_wait("Not all required input items for this module have been entered. Please check all fields.", title="Error", form_color='STANDOUT', wrap=True, wide=True)
            self.editing = True
        else:
            if self.parentApp.current_page <= (len(self.parentApp._display_pages) - 1):
                self.parentApp.current_page += 1
    
    def create(self):
        self.CANCEL_BUTTON_TEXT = "Previous"
        self.OK_BUTTON_TEXT = "Next"
        self.name = self.module._value["label"]

    def afterEditing(self):
        if self.parentApp.current_page < 0:
            self.parentApp.setNextForm("MODULE")
            self.parentApp.current_page = 0
        elif self.parentApp.current_page <= (len(self.parentApp._display_pages) - 1): 
            self.parentApp.setNextForm(self.parentApp._display_pages[self.parentApp.current_page])
        else:
            self.parentApp.setNextForm("SAVE")

#A button with the function of popping up a help message
class HelpButton(nps.ButtonPress):
    def __init__(self, screen, help_msg, *args, **keywords):
        self.help_msg = help_msg
        super(HelpButton, self).__init__(screen, *args, **keywords)
        
    def whenPressed(self):
        nps.notify_confirm(message=self.help_msg, title="Module/Submodule Descriptions", form_color='STANDOUT', wrap=True, wide=True, editw=1)

#Add/Remove Form button logic for submodule forms
class AddFormButton(nps.ButtonPress):
    def whenPressed(self):
        form = self.parent
        module_name = self.parent.module.name
        submodule_name = self.parent.submodule.name
        #Go through list of forms, and add it after all current forms of the same submodule
        found = False
        for i in range(0, len(form.parentApp.submodule_forms_data)):
            if (module_name == form.parentApp.submodule_forms_data[i]["module_name"]) & (submodule_name == form.parentApp.submodule_forms_data[i]["submodule_name"]):
                found = True
                copy_number = form.parentApp.submodule_forms_data[i]["copy_number"]+1
                insert_location = i+1
            elif found:
                break   
        #Create new form
        new_form = form.parentApp.createSubmoduleForm(module_name, submodule_name, copy_number)
        #Add to submodule forms list
        form.parentApp.submodule_forms_data.insert(insert_location, {"form":new_form, "submodule_name":submodule_name, "module_name":module_name, "copy_number":copy_number, "registered":False})
        #Register form and add to the list of formid's
        formid = module_name+"_"+submodule_name+"_"+str(copy_number)
        form.parentApp.registerForm(formid, new_form)
        form.parentApp._display_pages.insert(insert_location, formid)
        #Notify the user that it worked
        nps.notify_wait("New copy of submodule %s added as next page." % submodule_name, title="Submodule Duplicated", form_color='STANDOUT', wrap=True, wide=True)

class RemoveFormButton(nps.ButtonPress):
    def whenPressed(self):
        form = self.parent
        module_name = self.parent.module.name
        submodule_name = self.parent.submodule.name
        copy_number = self.parent.copy_number
        formid = module_name+"_"+submodule_name+"_"+str(copy_number)
        #Need to remove from the submodule_forms_data list and _display_pages list
        form.parentApp._display_pages.remove(formid)
        copy_count = 0
        for form_data in form.parentApp.submodule_forms_data:
            if (module_name == form_data["module_name"]) & (submodule_name == form_data["submodule_name"]) & (copy_number == form_data["copy_number"]):
                form.parentApp.submodule_forms_data.remove(form_data)
            elif (module_name == form_data["module_name"]) & (submodule_name == form_data["submodule_name"]):
                copy_count += 1
        #If the last one is removed, we need to unselect the IntroForm widget
        if copy_count == 0:
            ModuleForm = form.parentApp.getForm("MODULE")
            for widget_info in ModuleForm._widget_list:
                if module_name == widget_info["module_name"]:
                    for choice in widget_info["widget"].value:
                        if widget_info["widget"].values[choice] == submodule_name:
                            widget_info["widget"].value.remove(choice)
        nps.notify_wait("Deleted copy of submodule %s." % submodule_name, title="Submodule Removed", form_color='STANDOUT', wrap=True, wide=True)
        form.edit_return_value = True
        form.editing = False
        
#Custom Slider class that hits min and max properly
class FloatSlider(nps.Slider):
    def translate_value(self):
        stri = "%s" % self.value
        return stri

    def h_increase(self, ch):
        if (self.value + self.step <= self.out_of): self.value += self.step
        elif (self.value >= self.out_of - self.step) & (self.value <= self.out_of): self.value = self.out_of

    def h_decrease(self, ch):
        if (self.value - self.step >= self.lowest): self.value -= self.step
        elif (self.value <= self.step - self.lowest) & (self.value >= self.lowest): self.value = self.lowest

class TitleFloatSlider(nps.wgtitlefield.TitleText):
    _entry_type = FloatSlider

#The npyscreen class has a bug when trying to exit the widget
#Overriding the h_scroll_line_down method fixes that
class ExitPager(nps.Pager):
    def h_scroll_line_down(self, input):
        self.start_display_at += 1
        if self.scroll_exit and len(self._my_widgets)-self.cursor_line >= len(self.values)-1:
            self.editing = False
            self.how_exited = True
            
class TitleExitPager(nps.TitleMultiLine):
    _entry_type = ExitPager

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description='Launch the AXIOME UI')
    parser.add_argument("input", metavar="i", type=str, nargs='?', help="Input .ax file (optional)")
    args = parser.parse_args()
    #Pass the input file into the UI
    App = AXIOMEUI(args.input)
    App.run()   



#Taken from https://wiki.python.org/moin/Distutils/Tutorial

import os, sys, glob, fnmatch

## Added 10 Jan 2008
from distutils.core import setup, Extension
import distutils.command.install_data

## Code borrowed from wxPython's setup and config files
## Thanks to Robin Dunn for the suggestion.
## I am not 100% sure what's going on, but it works!
def opj(*args):
    path = os.path.join(*args)
    return os.path.normpath(path)

## Added 10 Jan 2008
# Specializations of some distutils command classes
class wx_smart_install_data(distutils.command.install_data.install_data):
    """need to change self.install_dir to the actual library dir"""
    def run(self):
        install_cmd = self.get_finalized_command('install')
        self.install_dir = getattr(install_cmd, 'install_lib')
        return distutils.command.install_data.install_data.run(self)

def find_data_files(srcdir, *wildcards, **kw):
    # get a list of all files under the srcdir matching wildcards,
    # returned in a format to be used for install_data
    def walk_helper(arg, dirname, files):
        if '.svn' in dirname:
            return
        names = []
        lst, wildcards = arg
        for wc in wildcards:
            wc_name = opj(dirname, wc)
            for f in files:
                filename = opj(dirname, f)

                if fnmatch.fnmatch(filename, wc_name) and not os.path.isdir(filename):
                    names.append(filename)
        if names:
            lst.append( (dirname, names ) )

    file_list = []
    recursive = kw.get('recursive', True)
    if recursive:
        os.path.walk(srcdir, walk_helper, (file_list, wildcards))
    else:
        walk_helper((file_list, wildcards),
                    srcdir,
                    [os.path.basename(f) for f in glob.glob(opj(srcdir, '*'))])
    return file_list

## This is a list of files to install, and where:
## Make sure the MANIFEST.in file points to all the right 
## directories too.
files = find_data_files('axiome/', '*.*')

from setuptools import setup

setup(name='axiome',
	version='2.0.1',
	description='AXIOME2: Automation Extension and Integration of Microbial Ecology',
	url='https://github.com/neufeld/AXIOME2',
	author='Michael Hall',
        author_email='hallm2533@gmail.com',
	classifiers=['Development Status :: 4 - Beta',
	'Environment :: Console',
	'Intended Audience :: Science/Research',
	'License :: OSI Approved :: MIT License',
	'Natural Language :: English',
	'Operating System :: POSIX :: Linux',
	'Topic :: Scientific/Engineering :: Bio-Informatics'],
        license='MIT',
	packages=['axiome'],
	install_requires=['npyscreen'],
	entry_points = {
        'console_scripts': ['axiome=axiome.axiome:main'],
    },
	data_files=files,
	## Borrowed from wxPython too:
	## Causes the data_files to be installed into the modules directory.
	## Override some of the default distutils command classes with my own.
	cmdclass = { 'install_data':    wx_smart_install_data },
	zip_safe=False)
